@@ -125,7 +125,7 @@ SEXP FARR_subset_assign_sequential_bare(
                 stop("Unsupported SEXP type");
             }
             }
-            region.flush();
+            // region.flush();
         } catch(std::exception &ex){
             stop("Error while writing sequential to partition " +
                 std::to_string(part + 1) + ". Reason: " + ex.what());
@@ -244,7 +244,26 @@ SEXP FARR_subset_assign_template(
     int64_t* idx1ptr0 = (int64_t*) REAL(idx1);
     const boost::interprocess::mode_t mode = boost::interprocess::read_write;
     
-    std::map<int64_t, boost::interprocess::file_mapping> conn_map;
+    std::map<int64_t, boost::interprocess::file_mapping*> conn_map;
+    std::map<int64_t, bool> conn_exist;
+    
+    std::map<int64_t, bool>::iterator it2;
+    
+    // it = conn_map.find(part);
+    for(IntegerVector::iterator partptr = partitions.begin();
+        partptr != partitions.end(); partptr++){
+        it2 = conn_exist.find(*partptr);
+        if(it2 == conn_exist.end()){
+            try {
+                std::string file = filebase + std::to_string(*partptr) + ".farr";
+                boost::interprocess::file_mapping *fm = new boost::interprocess::file_mapping(file.c_str(), mode);
+                conn_map.insert( std::pair<int64_t, boost::interprocess::file_mapping*>(*partptr, fm) );
+                conn_exist.insert( std::pair<int64_t, bool>(*partptr, true) );
+            } catch (...){
+                conn_exist.insert( std::pair<int64_t, bool>(*partptr, false) );
+            }
+        }
+    }
     
 #pragma omp parallel num_threads(ncores)
 {
@@ -284,11 +303,18 @@ SEXP FARR_subset_assign_template(
         std::string file = filebase + std::to_string(part) + ".farr";
         
         try{
-            boost::interprocess::file_mapping fm(file.c_str(), mode);
+            std::map<int64_t, boost::interprocess::file_mapping*>::const_iterator it = conn_map.find(part);
+            
+            if(it == conn_map.end()){
+                has_error = part;
+                error_msg = "Cannot open partition " + std::to_string(part + 1);
+                continue;
+            }
+            
             int64_t region_len = elem_size * (idx1_end - idx1_start + block_size * (idx2_end - idx2_start));
             int64_t region_offset = FARR_HEADER_LENGTH + elem_size * (block_size * idx2_start + idx1_start);
             boost::interprocess::mapped_region region(
-                    fm, mode, region_offset, region_len);
+                    *(it->second), mode, region_offset, region_len);
             region.advise(boost::interprocess::mapped_region::advice_sequential);
             
             char* begin = static_cast<char*>(region.get_address());
@@ -303,7 +329,7 @@ SEXP FARR_subset_assign_template(
                 idx1_start, idx2_start, 
                 idx2_ptr, idx2_len );
             
-            region.flush();
+            // region.flush();
         } catch(std::exception &ex){
             has_error = part;
             error_msg =  ex.what();
@@ -315,6 +341,11 @@ SEXP FARR_subset_assign_template(
     }
 }
 
+    std::map<int64_t, boost::interprocess::file_mapping*>::iterator it1 = conn_map.begin();
+    for(; it1 != conn_map.end(); it1++){
+        delete it1->second;
+    }
+
     // UNPROTECT(ncores);
     if( has_error >= 0 ){
         stop("Cannot write to partition " + 
